import json

from django.contrib import admin
from .models import Feedback


# Display an html table from a dict
# Credit: original author Rogerio Hilbert
def pretty_items(r, d, nametag="<strong>%s: </strong>", itemtag='<li>%s</li>\n',
                 valuetag="%s", blocktag=('<ul>', '</ul>\n')):
    if isinstance(d, dict):
        r.append(blocktag[0])
        for k, v in d.items():
            name = nametag % k
            if isinstance(v, dict) or isinstance(v, list):
                r.append(itemtag % name)
                pretty_items(r, v, nametag, itemtag, valuetag, blocktag)
            else:
                value = valuetag % v
                r.append(itemtag % (name + value))
        r.append(blocktag[1])
    elif isinstance(d, list):
        r.append(blocktag[0])
        for i in d:
            if isinstance(i, dict) or isinstance(i, list):
                r.append(itemtag % " - ")
                pretty_items(r, i, nametag, itemtag, valuetag, blocktag)
            else:
                r.append(itemtag % i)
        r.append(blocktag[1])


class FeedbackAdmin(admin.ModelAdmin):
    list_display = ("comment", "url", "screenshot_thumb", "user", "created")
    list_filter = ("created", "user", "url")
    search_fields = ("comment", "user__email", "user__name")
    readonly_fields = ("comment", "url", "user", "browser_html", "screenshot_thumb")
    exclude = ('browser', 'screenshot')
    ordering = ("-created",)

    def screenshot_thumb(self, feedback: Feedback):
        if feedback.screenshot:
            return u'<a href="%s" ><img src="%s" width="100"/></a>' % (feedback.screenshot.url, feedback.screenshot.url)
    screenshot_thumb.allow_tags = True
    screenshot_thumb.short_description = "Screenshot"

    def browser_html(self, feedback: Feedback):
        if feedback.browser:
            r = []
            pretty_items(r, json.loads(feedback.browser))
            return u''.join(r)
    browser_html.allow_tags = True
    browser_html.short_description = "Browser Info"

admin.site.register(Feedback, FeedbackAdmin)






from django.conf import settings
from django.db import models


# Create your models here.
class Feedback(models.Model):
    url = models.CharField(max_length=255)
    browser = models.TextField()
    comment = models.TextField()
    screenshot = models.ImageField(blank=True, null=True, upload_to="tellme/screenshots/")

    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.SET_NULL, blank=True, null=True)
    created = models.DateTimeField(auto_now_add=True)


# Receive the post_delete signal and delete the file associated with the model instance.
from django.db.models.signals import post_delete
from django.dispatch.dispatcher import receiver


@receiver(post_delete, sender=Feedback)
def feedback_screenshot_delete(sender, instance, **kwargs):
    # Pass false so FileField doesn't save the model.
    if instance.screenshot:
        instance.screenshot.delete(save=False)


import json
from base64 import b64decode

from django.http import JsonResponse, HttpResponseBadRequest
from django.core.files.base import ContentFile
from tellme.forms import FeedbackForm


def post_feedback(request):

    if request.method == 'POST' and request.is_ajax():

        # Copy Post data names into names used into the model in order to automatically create the model/form
        # from the request dicts
        feedback = json.loads(request.POST["feedback"])
        data = {'url': feedback['url'], 'browser': json.dumps(feedback['browser']), 'comment': feedback['note'],
                'user': request.user.id}
        imgstr = feedback['img'].split(';base64,')[1]
        file = {'screenshot': ContentFile(b64decode(imgstr), name="screenshot.png")}
        form = FeedbackForm(data, file)
        # check whether it's valid:
        if form.is_valid():
            form.save()
            return JsonResponse({})
        else:
            return JsonResponse({'error': dict(form.errors)})

    else:
        return HttpResponseBadRequest()



import os
from setuptools import setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-tellme',
    version='0.1',
    packages=['tellme'],
    include_package_data=True,
    license='BSD License',
    description='A simple Django app that enables user feedback.',
    long_description=README,
    url='https://github.com/ludrao/django-tellme',
    author='Ludrao',
    author_email='ludrao@ludrao.net',
    classifiers=[
	'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)
